@@ -1,8 +1,10 @@
 """Module for parsing conditions (statements that return boolean), called from command/_flow_control.py and lexer.py"""
 
-from dataclasses import dataclass
+from dataclasses import dataclass, field
 from typing import Union
 
+
+from ...compile.utils import is_connected
 from ..tokenizer import TokenType, Tokenizer, Token
 from ..exception import JMCSyntaxException, JMCValueError
 from ..datapack import DataPack
@@ -11,27 +13,26 @@ from .jmc_function import JMCFunction, FuncType
 
 from .builtin_function import bool_function
 
-AND_OPERATOR = '&&'
-OR_OPERATOR = '||'
-NOT_OPERATOR = '!'
+AND_OPERATOR = "&&"
+OR_OPERATOR = "||"
+NOT_OPERATOR = "!"
 
 IF = True
 UNLESS = False
 
-VAR = '__logic__'
+VAR = "__logic__"
 BOOL_FUNCTIONS = JMCFunction.get_subclasses(FuncType.BOOL_FUNCTION)
-count = 0
 
 
-@dataclass(eq=False, repr=True)
+@dataclass(eq=False, repr=True, slots=True)
 class Condition:
     """
     Dataclass for condition containing string representation(excluding if/unless) and whether it's for `if` or `unless`
     """
-    __slots__ = 'string', 'if_unless'
     string: str
     if_unless: bool
     """`True` means 'if', `False` means 'unless'"""
+    pre_commands: list[str] = field(default_factory=list)
 
     def reverse(self) -> None:
         self.if_unless = not self.if_unless
@@ -40,20 +41,22 @@ class Condition:
         return f"{'if' if self.if_unless else 'unless'} {self.string}"
 
 
-AST_TYPE = Union[dict[str,  # type: ignore
+AST_TYPE = dict[str,  # type: ignore
                       Union[str, list["AST_TYPE"], "AST_TYPE"]  # type: ignore
-                      ],
-                 Condition]
+                ] | Condition
 
 
-def merge_condition(conditions: list[Condition]) -> str:
+def merge_condition(conditions: list[Condition]) -> tuple[str, list[str]]:
     """
     Merge all condition into a single string for minecraft execute if
 
     :param conditions: List of conditions
-    :return: Minecraft arguments after `execute if`
+    :return: A tuple of (Minecraft arguments after `execute if`) and (List of precommands in strings)
     """
-    return ' '.join(str(condition) for condition in conditions)
+    precommands: list[str] = []
+    for condition in conditions:
+        precommands.extend(condition.pre_commands)
+    return (" ".join(str(condition) for condition in conditions), precommands)
 
 
 def custom_condition(
@@ -68,74 +71,66 @@ def custom_condition(
     """
     if tokens[0].token_type == TokenType.KEYWORD and tokens[0].string.startswith(
             DataPack.VARIABLE_SIGN):
-        tokens = tokenizer.split_keyword_tokens(tokens, ['>', '=', '<', '!'])
-        first_token = tokens[0]
-        for operator in ['===', '==',
-                         '>=', '<=', '!=', '>', '<', '=']:  # sort key=len
-            list_of_tokens = tokenizer.find_tokens(tokens, operator)
-            if len(list_of_tokens) == 1:
-                continue
-            elif len(list_of_tokens) > 2:
-                raise JMCSyntaxException(
-                    f"Duplicated operator({operator}) in condition", list_of_tokens[2][-1], tokenizer)
 
-            if len(list_of_tokens[0]) > 1:
-                raise JMCSyntaxException(
-                    f"Unexpected token ('{list_of_tokens[0][1].string}') after variable ('{list_of_tokens[0][0].string}') in condition", list_of_tokens[0][1], tokenizer, suggestion="Expected operator")
+        if len(tokens) == 1:
+            raise JMCSyntaxException(
+                "Operator not found in custom condition", tokens[0], tokenizer)
+        if len(tokens) == 2:
+            raise JMCSyntaxException(
+                f"Expected token after operator{tokens[1].string} in custom condition (got nothing)", tokens[0], tokenizer)
+        if len(tokens) > 3:
+            raise JMCSyntaxException(
+                f"Unexpected token ('{tokens[3].string}') after variable ('{tokens[2].string}') in condition", tokens[3], tokenizer)
 
-            if len(list_of_tokens[1]) > 1:
-                raise JMCSyntaxException(
-                    "Unexpected token in condition", list_of_tokens[1][-1], tokenizer)
-
-            second_token = list_of_tokens[1][0]
+        first_token, operator_token, second_token = tokens
+        if operator_token.token_type == TokenType.OPERATOR:
             scoreboard_player = find_scoreboard_player_type(
                 second_token, tokenizer)
+            operator = operator_token.string
 
             if scoreboard_player.player_type == PlayerType.INTEGER:
                 if not isinstance(scoreboard_player.value, int):
                     raise ValueError("scoreboard_player.value is not int")
                 compared = f'score {first_token.string} {DataPack.var_name} matches'
-                if operator in {'===', '==', '='}:
+                if operator in {"===", "==", "="}:
                     return Condition(
                         f'{compared} {scoreboard_player.value}', IF)
-                if operator == '>=':
+                if operator in {"!=", "!=="}:
+                    return Condition(
+                        f'{compared} {scoreboard_player.value}', UNLESS)
+                if operator == ">=":
                     return Condition(
                         f'{compared} {scoreboard_player.value}..', IF)
-                if operator == '>':
+                if operator == ">":
                     return Condition(
                         f'{compared} {scoreboard_player.value+1}..', IF)
-                if operator == '<=':
+                if operator == "<=":
                     return Condition(
                         f'{compared} ..{scoreboard_player.value}', IF)
-                if operator == '<':
+                if operator == "<":
                     return Condition(
                         f'{compared} ..{scoreboard_player.value-1}', IF)
+                raise JMCSyntaxException(
+                    f"Unrecognized operator ({operator})", operator_token, tokenizer)
+
             else:
-                if operator == '!=':
+                if operator == "!=":
                     if isinstance(scoreboard_player.value, int):
                         raise ValueError("scoreboard_player.value is int")
                     return Condition(
                         f'score {first_token.string} {DataPack.var_name} = {scoreboard_player.value[1]} {scoreboard_player.value[0]}', UNLESS)
 
-                if operator in {'===', '==', '='}:
+                if operator in {"===", "==", "="}:
                     operator = '='
 
                 if isinstance(scoreboard_player.value, int):
                     raise ValueError("scoreboard_player.value is int")
                 return Condition(
                     f'score {first_token.string} {DataPack.var_name} {operator} {scoreboard_player.value[1]} {scoreboard_player.value[0]}', IF)
-            break
 
-        if tokens[1].token_type == TokenType.KEYWORD and tokens[1].string == 'matches':
-            if len(tokens) > 3:
-                raise JMCSyntaxException(
-                    "Unexpected token in condition", tokens[3], tokenizer)
-            if tokens[2].token_type != TokenType.KEYWORD:
-                raise JMCSyntaxException(
-                    "Expected keyword", tokens[2], tokenizer)
-
-            match_tokens_ = tokenizer.split_keyword_token(tokens[2], '..')
-            match_tokens = tokenizer.find_token(match_tokens_, '..')
+        elif operator_token.token_type == TokenType.KEYWORD and operator_token.string == "matches":
+            match_tokens_ = tokenizer.split_keyword_token(tokens[2], "..")
+            match_tokens = tokenizer.find_token(match_tokens_, "..")
             if len(match_tokens) != 2 or len(
                     match_tokens[0]) > 1 or len(match_tokens[1]) > 1:
                 raise JMCSyntaxException(
@@ -159,8 +154,9 @@ def custom_condition(
             return Condition(
                 f'score {first_token.string} {DataPack.var_name} matches {tokens[2].string}', IF)
 
-        raise JMCSyntaxException(
-            "Operator not found in custom condition", tokens[0], tokenizer)
+        else:
+            raise JMCSyntaxException(
+                f"Expected operator or 'matches' (got {tokens[1].token_type.value})", tokens[1], tokenizer)
 
     matched_function = BOOL_FUNCTIONS.get(
         tokens[0].string, None)
@@ -172,6 +168,7 @@ def custom_condition(
         return Condition(
             *matched_function(tokens[1], datapack, tokenizer).call_bool())
     # End
+
     conditions: list[str] = []
     if tokens[0].string not in {"block", "blocks",
                                 "data", "entity", "predicate", "score"}:
@@ -180,15 +177,21 @@ def custom_condition(
             tokens[0],
             tokenizer,
             suggestion="Consider using 'block' or 'blocks' or 'data' or 'entity' or 'predicate' or 'score'.")
+
+    last_token = tokens[0]
     for token in tokens:
         if token.token_type == TokenType.PAREN_SQUARE:
             if not conditions:
                 raise JMCSyntaxException(
                     "Unexpected square parenthesis []", token, tokenizer)
             conditions[-1] += token.string
+        elif is_connected(
+                token, last_token):
+            conditions[-1] += token.string
         else:
             conditions.append(token.string)
-    return Condition(' '.join(conditions), IF)
+        last_token = token
+    return Condition(" ".join(conditions), IF)
 
 
 def find_operator(_tokens: list[Token], operator: str,
@@ -203,16 +206,16 @@ def find_operator(_tokens: list[Token], operator: str,
     """
     list_of_tokens: list[list[Token]] = []
     tokens: list[Token] = []
-    if _tokens[0].token_type == TokenType.KEYWORD and _tokens[0].string == operator:
+    if _tokens[0].token_type == TokenType.OPERATOR and _tokens[0].string == operator:
         raise JMCSyntaxException(
             f"Unexpected operator ({operator})", _tokens[0], tokenizer)
 
-    elif _tokens[-1].token_type == TokenType.KEYWORD and _tokens[-1].string == operator:
+    elif _tokens[-1].token_type == TokenType.OPERATOR and _tokens[-1].string == operator:
         raise JMCSyntaxException(
             f"Unexpected operator ({operator})", _tokens[-1], tokenizer)
 
     for token in _tokens:
-        if token.token_type == TokenType.KEYWORD and token.string == operator:
+        if token.token_type == TokenType.OPERATOR and token.string == operator:
             list_of_tokens.append(tokens)
             tokens = []
         else:
@@ -233,32 +236,25 @@ def condition_to_ast(
     :return: Abstract syntax tree
     """
     if len(tokens) == 1 and tokens[0].token_type == TokenType.PAREN_ROUND:
-        if tokens[0].string == '()':
+        if tokens[0].string == "()":
             raise JMCSyntaxException(
-                f"Empty round parenthesis () inside condition", tokens[0], tokenizer)
+                "Empty round parenthesis () inside condition", tokens[0], tokenizer)
 
         tokenizer = Tokenizer(tokens[0].string[1:-1], tokenizer.file_path,
                               tokens[0].line, tokens[0].col + 1, tokenizer.file_string, expect_semicolon=False)
         tokens = tokenizer.programs[0]
-    tokens = tokenizer.split_keyword_tokens(tokens, [OR_OPERATOR])
     list_of_tokens = find_operator(tokens, OR_OPERATOR, tokenizer)
     if len(list_of_tokens) > 1:
         return {"operator": OR_OPERATOR, "body": [
             condition_to_ast(tokens, tokenizer, datapack) for tokens in list_of_tokens]}
-    else:
-        tokens = list_of_tokens[0]
 
-    tokens = tokenizer.split_keyword_tokens(tokens, [AND_OPERATOR])
     list_of_tokens = find_operator(tokens, AND_OPERATOR, tokenizer)
     if len(list_of_tokens) > 1:
         return {"operator": AND_OPERATOR, "body": [
             condition_to_ast(tokens, tokenizer, datapack) for tokens in list_of_tokens]}
-    else:
-        tokens = list_of_tokens[0]
 
     # NotOperator should have a body as either dict or string and not list
-    tokens = tokenizer.split_keyword_tokens(tokens, [NOT_OPERATOR])
-    if tokens[0].token_type == TokenType.KEYWORD and tokens[0].string == NOT_OPERATOR:
+    if tokens[0].token_type == TokenType.OPERATOR and tokens[0].string == NOT_OPERATOR:
         return {"operator": NOT_OPERATOR, "body":
                 condition_to_ast(tokens[1:], tokenizer, datapack)}
 
@@ -266,7 +262,7 @@ def condition_to_ast(
 
 
 def ast_to_commands(
-        ast: AST_TYPE) -> tuple[list[Condition], list[tuple[list[Condition], int]] | None]:
+        ast: AST_TYPE, datapack: DataPack) -> tuple[list[Condition], list[tuple[list[Condition], int]] | None]:
     """
     Parse abstract syntax tree into list of conditions and list of commands that need to come before for it to works
 
@@ -274,36 +270,41 @@ def ast_to_commands(
     :raises ValueError: Invalid AST
     :return: A tuple of (
         A chain of conditions(List of Condition)
-
+        and
         Commands(
             List of Condition and n (`__logic__n` for minecraft function name)
         ) that need to come before (can be None)
     )
     """
-    global count
     if isinstance(ast, Condition):
         return [ast], None
 
     if ast["operator"] == AND_OPERATOR:
         conditions: list[Condition] = []
         precommand_and: list[tuple[list[Condition], int]] = []
-        for body in ast["body"]:  # type: ignore
-            if isinstance(body, str):
+        if isinstance(ast["body"], Condition):
+            raise ValueError(
+                'ast["body"] is a Condition instead of list in AND')
+        for and_body in ast["body"]:
+            if isinstance(and_body, str):
                 raise ValueError('ast["body"] is string')
-            _conditions, precommand = ast_to_commands(body)  # noqa
+            _conditions, precommand = ast_to_commands(and_body, datapack)  # noqa
             conditions.extend(_conditions)
             if precommand is not None:
                 precommand_and.extend(precommand)
         return conditions, (precommand_and if precommand_and else None)
 
     elif ast["operator"] == OR_OPERATOR:
-        _count = count
-        count += 1
+        _count = datapack.data.condition_count
+        datapack.data.condition_count += 1
         precommand_or: list[tuple[list[Condition], int]] = []
-        for body in ast["body"]:  # type: ignore
-            if isinstance(body, str):
+        if isinstance(ast["body"], Condition):
+            raise ValueError(
+                'ast["body"] is a Condition instead of list in OR')
+        for or_body in ast["body"]:
+            if isinstance(or_body, str):
                 raise ValueError('ast["body"] is string')
-            conditions, precommand = ast_to_commands(body)
+            conditions, precommand = ast_to_commands(or_body, datapack)
             if precommand is not None:
                 precommand_or.extend(precommand)
             precommand_or.append((conditions, _count))
@@ -312,10 +313,16 @@ def ast_to_commands(
             f"score {VAR}{_count} {DataPack.var_name} matches 1", IF)], precommand_or
 
     elif ast["operator"] == NOT_OPERATOR:
-        body = ast["body"]
-        if not isinstance(ast["body"], Condition):
-            raise ValueError('ast["body"] is noy Condition')
-        conditions, precommand = ast_to_commands(ast["body"])
+        if isinstance(ast["body"], Condition):
+            conditions, precommand = ast_to_commands(ast["body"], datapack)
+        elif isinstance(ast["body"], dict):
+            if ast["body"]["operator"] == OR_OPERATOR:
+                ast["body"]["operator"] = AND_OPERATOR
+            elif ast["body"]["operator"] == AND_OPERATOR:
+                ast["body"]["operator"] = OR_OPERATOR
+            conditions, precommand = ast_to_commands(ast["body"], datapack)
+        else:
+            raise ValueError('ast["body"] is a list or str')
         for condition in conditions:
             condition.reverse()
         return conditions, precommand
@@ -323,7 +330,7 @@ def ast_to_commands(
     raise ValueError("Invalid AST")
 
 
-def ast_to_strings(ast: AST_TYPE) -> tuple[str, str]:
+def ast_to_strings(ast: AST_TYPE, datapack: DataPack) -> tuple[str, str]:
     """
     Turns AST into tuple of full `execute if command` a multiple line string representing precommands
 
@@ -331,26 +338,33 @@ def ast_to_strings(ast: AST_TYPE) -> tuple[str, str]:
     :return: tuple of `execute if` command(excluding `execute`) a multiple line string representing precommands
     """
 
-    conditions, precommand_conditions = ast_to_commands(ast)
+    conditions, precommand_conditions = ast_to_commands(ast, datapack)
     if precommand_conditions is None:
         precommand = ""
     else:
-        precommands = []
+        precommands: list[str] = []
         current_count = -1
         for conditions_and_count in precommand_conditions:
             if conditions_and_count[1] > current_count:
                 current_count += 1
                 precommands.append(
-                    f"scoreboard players reset {VAR}{current_count} {DataPack.var_name}")
+                    f"scoreboard players set {VAR}{current_count} {DataPack.var_name} 0")
+                merged_condition, merged_condition_pre_command = merge_condition(
+                    conditions_and_count[0])
+                precommands.extend(merged_condition_pre_command)
                 precommands.append(
-                    f"execute {merge_condition(conditions_and_count[0])} run scoreboard players set {VAR}{current_count} {DataPack.var_name} 1")
+                    f"execute {merged_condition} run scoreboard players set {VAR}{current_count} {DataPack.var_name} 1")
                 continue
 
+            merged_condition, merged_condition_pre_command = merge_condition(
+                conditions_and_count[0])
+            precommands.extend(merged_condition_pre_command)
             precommands.append(
-                f"execute unless score {VAR}{current_count} {DataPack.var_name} matches 1 {merge_condition(conditions_and_count[0])} run scoreboard players set {VAR}{current_count} {DataPack.var_name} 1")
-        precommand = '\n'.join(precommands)
+                f"execute unless score {VAR}{current_count} {DataPack.var_name} matches 1 {merged_condition} run scoreboard players set {VAR}{current_count} {DataPack.var_name} 1")
+        precommand = "\n".join(precommands)
 
-    condition_string = merge_condition(conditions)
+    condition_string, precommands_ = merge_condition(conditions)
+    precommand += "\n".join(precommands_)
     return condition_string, precommand
 
 
@@ -374,12 +388,11 @@ def parse_condition(condition_token: Token |
     :param datapack: Datapack object
     :return: tuple of `execute if` command(excluding `execute`) a multiple line string representing precommands
     """
-    global count
-    count = 0
+    datapack.data.condition_count = 0
     tokens = condition_token if isinstance(
         condition_token, list) else [condition_token]
 
     ast = condition_to_ast(tokens, tokenizer, datapack)
-    condition, precommand = ast_to_strings(ast)
-    precommand = precommand + '\n' if precommand else ""
+    condition, precommand = ast_to_strings(ast, datapack)
+    precommand = precommand + "\n" if precommand else ""
     return condition, precommand
