@@ -1,14 +1,14 @@
 from collections import defaultdict
 from enum import Enum, auto
 from json import JSONDecodeError, loads
-from typing import Any, Callable
+from typing import TYPE_CHECKING, Any, Callable
 
-from jmc.compile.utils import convention_jmc_to_mc
-
-from .utils import ArgType, find_scoreboard_player_type, verify_args, Arg
+from ...compile.utils import convention_jmc_to_mc, is_float
+from ...compile.datapack_data import Item
+from .utils import ArgType, FormattedText, NumberType, find_scoreboard_player_type, verify_args, Arg
 from ..datapack import DataPack, Function
 from ..exception import JMCDecodeJSONError, JMCMissingValueError, JMCValueError
-from ..tokenizer import Token, Tokenizer
+from ..tokenizer import Token, TokenType, Tokenizer
 
 
 class FuncType(Enum):
@@ -51,11 +51,11 @@ class JMCFunction:
     :raises JMCValueError: Missing required positional argument
     :raises NotImplementedError: Call function not implemented
     """
-    __slots__ = ('arg_type', 'func_type', 'name',
-                 'call_string', 'defaults', '_ignore',
-                 'token', 'datapack', 'tokenizer',
-                 'is_execute', 'var', 'args',
-                 'raw_args')
+    __slots__ = ("arg_type", "func_type", "name",
+                 "call_string", "defaults", "_ignore",
+                 "token", "datapack", "tokenizer",
+                 "is_execute", "var", "args",
+                 "raw_args", "number_type")
     _decorated: bool = False
     """A private attribute that will be changed by a decorator to check for missing decorator (Set by decorator)"""
     arg_type: dict[str, ArgType]
@@ -68,6 +68,8 @@ class JMCFunction:
     """String that will be used in .jmc to call the function (Set by decorator)"""
     defaults: dict[str, str]
     """Defaults value of each parameter as string (Set by decorator)"""
+    number_type: dict[str, NumberType]
+    """Dictionary containing some number parameter that need to be specific (Set by decorator)"""
     _ignore: set[str]
     """Private set of parameter for parser to ignore and leave as is (Set by decorator)"""
 
@@ -125,15 +127,14 @@ class JMCFunction:
             if key in self._ignore:
                 pass
             elif arg.arg_type == ArgType._FUNC_CALL:
-                self.args[
-                    key] = f"function {datapack.namespace}:{convention_jmc_to_mc(arg.token, self.tokenizer)}"
+                if ":" in arg.token.string:
+                    self.args[key] = f"function {arg.token.string}"
+                else:
+                    self.args[
+                        key] = f"function {datapack.namespace}:{convention_jmc_to_mc(arg.token, self.tokenizer)}"
             elif arg.arg_type == ArgType.ARROW_FUNC:
-                self.args[key] = '\n'.join(
+                self.args[key] = "\n".join(
                     datapack.parse_function_token(arg.token, tokenizer))
-            elif arg.arg_type == ArgType.INTEGER:
-                self.args[key] = arg.token.string
-                # self.args[key] = str(find_scoreboard_player_type(
-                #     arg.token, tokenizer).value)
             elif arg.arg_type == ArgType.FLOAT:
                 self.args[key] = str(float(arg.token.string))
             elif arg.arg_type in {ArgType.SCOREBOARD_PLAYER, ArgType.SCOREBOARD}:
@@ -146,6 +147,35 @@ class JMCFunction:
             else:
                 self.args[key] = arg.token.string
 
+        for parameter, number_type in self.number_type.items():
+            if self.arg_type[parameter] == ArgType.SCOREBOARD_PLAYER:
+                if not is_float(self.args[parameter]):
+                    continue
+
+            elif self.arg_type[parameter] not in {
+                    ArgType.INTEGER, ArgType.FLOAT}:
+                raise ValueError(f"{parameter} paremeter is not number")
+
+            if number_type == NumberType.POSITIVE:
+                if float(self.args[parameter]) <= 0:
+                    raise JMCValueError(
+                        f"{parameter} can only be {number_type.value}",
+                        self.raw_args[parameter].token,
+                        tokenizer)
+            elif number_type == NumberType.ZERO_POSITIVE:
+                if float(self.args[parameter]) < 0:
+                    raise JMCValueError(
+                        f"{parameter} can only be {number_type.value}",
+                        self.raw_args[parameter].token,
+                        tokenizer)
+
+            elif number_type == NumberType.NON_ZERO:
+                if float(self.args[parameter]) == 0:
+                    raise JMCValueError(
+                        f"{parameter} can only be {number_type.value}",
+                        self.raw_args[parameter].token,
+                        tokenizer)
+
         self.__post__init__()
 
     def __post__init__(self) -> None:
@@ -153,6 +183,74 @@ class JMCFunction:
         This function will be called after initialization of the object
         """
 
+    def create_new_item(
+            self, item: "Item", modify_nbt: dict[str, Token] | None = None) -> "Item":
+        item_type = item.item_type
+        nbt = item.raw_nbt
+        if modify_nbt is None:
+            modify_nbt = {}
+        for key, value_token in modify_nbt.items():
+            if key in nbt:
+                raise JMCValueError(
+                    f"{key} is already inside the nbt",
+                    value_token,
+                    self.tokenizer)
+
+            nbt[key] = value_token
+        return Item(
+            item_type,
+            self.datapack.token_dict_to_raw_js_object(nbt, self.tokenizer),
+            nbt
+        )
+
+    def create_item(self, item_type_param: str = "itemType", display_name_param: str = "displayName",
+                    lore_param: str = "lore", nbt_param: str = "nbt", modify_nbt: dict[str, Token] | None = None) -> "Item":
+        if modify_nbt is None:
+            modify_nbt = {}
+
+        item_type = self.args[item_type_param]
+        if item_type.startswith("minecraft:"):
+            item_type = item_type[10:]
+        if self.args[lore_param]:
+            lores = self.datapack.parse_list(
+                self.raw_args[lore_param].token, self.tokenizer, TokenType.STRING)
+        else:
+            lores = []
+        nbt = self.tokenizer.parse_js_obj(
+            self.raw_args[nbt_param].token) if self.args[nbt_param] else {}
+
+        for key, value_token in modify_nbt.items():
+            if key in nbt:
+                raise JMCValueError(
+                    f"{key} is already inside the nbt",
+                    value_token,
+                    self.tokenizer)
+
+            nbt[key] = value_token
+
+        lore_ = ",".join([repr(str(FormattedText(lore, self.raw_args[lore_param].token, self.tokenizer, self.datapack, is_default_no_italic=True, is_allow_score_selector=False)))
+                         for lore in lores])
+
+        if self.args[display_name_param]:
+            if "display" in nbt:
+                raise JMCValueError(
+                    "display is already inside the nbt",
+                    self.token,
+                    self.tokenizer)
+
+            nbt["display"] = Token.empty(f"""{{Name:{repr(
+                self.format_text(
+                display_name_param,
+                is_default_no_italic=True,
+                is_allow_score_selector=False)
+                )},Lore:[{lore_}]}}""")
+
+        return Item(
+            item_type,
+            self.datapack.token_dict_to_raw_js_object(nbt, self.tokenizer),
+            nbt
+        )
+
     def call(self) -> str:
         """
         This function will be called when user call matching JMC custom function
@@ -162,12 +260,12 @@ class JMCFunction:
         """
         raise NotImplementedError("Call function not implemented")
 
-    def call_bool(self) -> tuple[str, bool]:
+    def call_bool(self) -> tuple[str, bool, list[str]]:
         """
         This function will be called when user call matching JMC boolean(Can only be used in condition) function
 
         :raises NotImplementedError: When the subclass's call method is not implemented
-        :return: Tuple of Minecraft command as string and boolean(True->if, False->unless)
+        :return: Tuple of Minecraft command as string and boolean(True->if, False->unless) and list of precommands
         """
         raise NotImplementedError("Call(boolean) function not implemented")
 
@@ -186,18 +284,22 @@ class JMCFunction:
 
         return cls.__subcls[func_type]
 
-    def is_never_used(self, call_string: str | None = None) -> bool:
+    def is_never_used(self, call_string: str | None = None,
+                      parameters: list[str] | None = None) -> bool:
         """
-        Add current function to datapack.used_command and return whether it's already there
+        Add current function to datapack.used_command and return whether it's not already there
 
-        :param: Any string instead of default call string for more specific check. For example, minecraft command
-        :return: Whether this function has been called by the user before
+        :param call_string: Any string instead of default call string for more specific check. For example, minecraft command, defaults to self.call_string
+        :param parameters: Details of call_string
+        :return: Whether this function has never been called by the user before
         """
         if call_string is None:
             call_string = self.call_string
-        is_in = call_string not in self.datapack.used_command
+        is_not_in = call_string not in self.datapack.used_command
+        if parameters is not None:
+            call_string = call_string + '/' + '/'.join(parameters)
         self.datapack.used_command.add(call_string)
-        return is_in
+        return is_not_in
 
     def get_private_function(self, function_name: str) -> Function:
         """
@@ -208,6 +310,26 @@ class JMCFunction:
         """
         return self.datapack.private_functions[self.name][function_name]
 
+    def format_text(self, parameter: str, is_default_no_italic: bool = False,
+                    is_allow_score_selector: bool = True) -> str:
+        """
+        Get FormattedText string from an argument
+
+        :param parameter: Parameter
+        :param is_default_no_italic: Whether to set italic to False by default, defaults to False
+        :param is_allow_score_selector: Whether to allow score selector in the string, defaults to True
+        :return: Raw json string
+        """
+        return str(
+            FormattedText(
+                self.args[parameter],
+                self.raw_args[parameter].token,
+                self.tokenizer,
+                self.datapack,
+                is_default_no_italic=is_default_no_italic,
+                is_allow_score_selector=is_allow_score_selector)
+        )
+
     def make_empty_private_function(self, function_name: str) -> Function:
         """
         Make private function with no content
@@ -231,11 +353,18 @@ class JMCFunction:
             json = loads(self.args[parameter])
         except JSONDecodeError as error:
             raise JMCDecodeJSONError(
-                error, self.raw_args[parameter].token, self.tokenizer)
+                error, self.raw_args[parameter].token, self.tokenizer) from error
 
         return json
 
     def check_bool(self, parameter: str) -> bool:
+        """
+        Check if argument is true or false
+
+        :param parameter: String parameter
+        :raises JMCValueError: Argument is invalid
+        :return: Whether argument is true
+        """
         if self.args[parameter] not in {"true", "false"}:
             raise JMCValueError(
                 f"'{parameter}' only accepts true or false in {self.call_string}",
@@ -245,7 +374,7 @@ class JMCFunction:
 
 
 def func_property(func_type: FuncType, call_string: str, name: str, arg_type: dict[str, ArgType], defaults: dict[str, str] = {
-}, ignore: set[str] = set()) -> Callable[[type[JMCFunction]], type[JMCFunction]]:
+}, ignore: set[str] = set(), number_type: dict[str, NumberType] = {}) -> Callable[[type[JMCFunction]], type[JMCFunction]]:
     """
     Decorator factory for setting property of custom JMC function
 
@@ -254,7 +383,8 @@ def func_property(func_type: FuncType, call_string: str, name: str, arg_type: di
     :param name: String for the function's name, for further use
     :param arg_type: Dictionary containing all parameter and it's ArgType
     :param defaults: Defaults value of each parameter as string, defaults to {}
-    :param ignore: Set of parameter for parser to ignore and leave as is
+    :param ignore: Set of parameter for parser to ignore and leave as is, defaults to set()
+    :param number_type: Dictionary containing some number parameter that need to be specific, defaults to {}
     :return: A decorator for JMCFunction class
     """
     def decorator(cls: type[JMCFunction]) -> type[JMCFunction]:
@@ -268,6 +398,7 @@ def func_property(func_type: FuncType, call_string: str, name: str, arg_type: di
         cls.call_string = call_string
         cls.arg_type = arg_type
         cls.defaults = defaults
+        cls.number_type = number_type
         # for default in defaults:
         #     if default not in arg_type:
         #         raise BaseException()
