@@ -1,3 +1,4 @@
+"""Module handling datapack"""
 from collections import defaultdict
 from typing import TYPE_CHECKING, Any, Iterable
 from json import JSONEncoder, dumps
@@ -15,7 +16,7 @@ if TYPE_CHECKING:
 logger = Logger(__name__)
 
 
-NEW_LINE = '\n'
+NEW_LINE = "\n"
 
 
 class FunctionEncoder(JSONEncoder):
@@ -35,7 +36,7 @@ class Function:
 
     :param commands: List of minecraft commands(string), defaults to empty list
     """
-    __slots__ = 'commands'
+    __slots__ = ("commands", )
     commands: list[str]
 
     def __init__(self, commands: list[str] | None = None) -> None:
@@ -90,7 +91,7 @@ class Function:
 
         :return: Commands of the function in form of a single string
         """
-        return '\n'.join(self.commands)
+        return "\n".join(self.commands)
 
     @property
     def length(self) -> int:
@@ -112,27 +113,13 @@ class Function:
 
     def __split(self, strings: list[str]) -> list[str]:
         """
-        Loop through every line in each string of command(s) and make a new list with every element having only 1 line of command while optimizing every command
+        Loop through every line in each string of command(s) and make a new list with every element having only 1 line of command
 
         :param strings: minecraft commands(strings),each string can have multiple lines
         :return: minecraft commands(strings),each string can have only a single line
         """
-        return [self.optimize(
-            str_) for string in strings for str_ in string.split('\n') if str_]
-
-    def optimize(self, string: str) -> str:
-        """
-        Optimize minecraft command by remove redundancy
-
-        :param string: A minecraft command
-        :return: An optimized minecraft command
-
-        .. todo:: Finish optimization
-        """
-        if string.startswith('execute'):
-            if string.startswith('execute run '):
-                string = string[12:]  # len('execute run ') = 11
-        return string
+        return [
+            line for string in strings for line in string.split("\n") if line]
 
 
 class DataPack:
@@ -142,23 +129,20 @@ class DataPack:
     :param namespace: Datapack's namespace
     :param lexer: Lexer object
     """
-    __slot__ = ('PRIVATE_NAME', 'LOAD_NAME', 'TICK_NAME',
-                'VAR_NAME', 'INT_NAME', 'VARIABLE_SIGN',
-                'data', '_tick_json', 'ints',
-                'functions', 'load_function', 'jsons',
-                'private_functions', 'private_function_count',
-                '__scoreboards', 'loads', 'ticks', 'namespace',
-                'used_command', 'lexer')
-    private_name = '__private__'
-    load_name = '__load__'
-    tick_name = '__tick__'
-    var_name = '__variable__'
-    int_name = '__int__'
-    VARIABLE_SIGN = '$'
+    __slot__ = ("data", "ints",
+                "functions", "load_function", "jsons",
+                "private_functions", "private_function_count",
+                "__scoreboards", "loads", "ticks", "namespace",
+                "used_command", "lexer", "defined_file_pos", "after_ticks", "after_loads")
+    private_name = "__private__"
+    load_name = "__load__"
+    tick_name = "__tick__"
+    var_name = "__variable__"
+    int_name = "__int__"
+    storage_name = "__storage__"
+    VARIABLE_SIGN = "$"
     """Data read from header file(s)"""
 
-    _tick_json = None
-
     def __init__(self, namespace: str, lexer: "Lexer") -> None:
         logger.debug("Initializing Datapack")
         self.ints: set[int] = set()
@@ -175,8 +159,7 @@ class DataPack:
         self.private_function_count: dict[str, int] = defaultdict(int)
         """Current count of how many private functions there are in each group name"""
         self.__scoreboards: dict[str, str] = {
-            self.var_name: 'dummy',
-            self.int_name: 'dummy'
+            self.var_name: "dummy"
         }
         """Minecraft scoreboards that are going to be created"""
 
@@ -184,6 +167,10 @@ class DataPack:
         """Output list of commands for load"""
         self.ticks: list[str] = []
         """Output list of commands for tick"""
+        self.after_loads: list[str] = []
+        """Output list of commands at the end of load"""
+        self.after_ticks: list[str] = []
+        """Output list of commands at the end of tick"""
         self.namespace = namespace
         """Datapack's namespace"""
 
@@ -196,7 +183,10 @@ class DataPack:
         self.data = Data()
         """Extra information that can be shared across all JMC function"""
 
-    def add_objective(self, objective: str, criteria: str = 'dummy') -> None:
+        self.defined_file_pos: dict[str, tuple[Token, Tokenizer]] = {}
+        """Dictionary of mcfunction or json path and it's first defined token and tokenizer"""
+
+    def add_objective(self, objective: str, criteria: str = "dummy") -> None:
         """
         Add minecraft scoreboard objective
 
@@ -253,7 +243,7 @@ class DataPack:
         self.jsons[f"{json_type}/{name}"] = json
 
     def add_arrow_function(self, name: str, token: Token,
-                           tokenizer: Tokenizer) -> str:
+                           tokenizer: Tokenizer, force_create_func: bool = False) -> str:
         """
         Add private function for user (arrow function)
 
@@ -263,16 +253,17 @@ class DataPack:
         :raises JMCSyntaxWarning: If the string in curly bracket is empty
         :return: Minecraft function call string
         """
-        if token.string == '{}':
+        if token.string == "{}":
             raise JMCSyntaxWarning("Empty function", token, tokenizer)
 
         commands = self.parse_function_token(token, tokenizer)
-        if len(commands) == 1 and NEW_LINE not in commands[0]:
+        if not force_create_func and len(
+                commands) == 1 and NEW_LINE not in commands[0]:
             return commands[0]
-        else:
-            count = self.get_count(name)
-            self.private_functions[name][count] = Function(commands)
-            return self.call_func(name, count)
+
+        count = self.get_count(name)
+        self.private_functions[name][count] = Function(commands)
+        return self.call_func(name, count)
 
     def add_custom_private_function(self, name: str, token: Token, tokenizer: Tokenizer, count: str,
                                     precommands: list[str] | None = None, postcommands: list[str] | None = None) -> str:
@@ -298,6 +289,25 @@ class DataPack:
         self.private_functions[name][count] = Function(commands)
         return self.call_func(name, count)
 
+    def add_private_function(self, name: str, command: str,
+                             count: str | None = None) -> str:
+        """
+        Add private function but don't create new function unless it's neccessary
+
+        :param name: Name of the private function's group
+        :param command: Multiline minecraft command
+        :param count: Name of the function (usually as count), defaults to current count + 1
+        :return: Minecraft function call string / The command itself
+        """
+
+        if "\n" not in command:
+            return command
+
+        if count is None:
+            count = self.get_count(name)
+        self.private_functions[name][count] = Function([command])
+        return self.call_func(name, count)
+
     def add_raw_private_function(
             self, name: str, commands: list[str], count: str | None = None) -> str:
         """
@@ -325,13 +335,38 @@ class DataPack:
         return self.lexer.parse_func_content(
             token.string[1:-1], tokenizer.file_path, token.line, token.col, tokenizer.file_string)
 
-    def add_tick_command(self, command: str) -> None:
-        self.ticks.append(command)
+    def add_tick_command(self, command: str, *,
+                         is_after: bool = False) -> None:
+        """
+        Add command to self.ticks or self.after_ticks
 
-    def add_load_command(self, command: str) -> None:
-        self.loads.append(command)
+        :param command: Minecraft command string
+        :param is_after: Whether to add it to after_ticks instead
+        """
+        if is_after:
+            self.after_ticks.append(command)
+        else:
+            self.ticks.append(command)
+
+    def add_load_command(self, command: str, *,
+                         is_after: bool = False) -> None:
+        """
+        Add command to self.loads or self.after_loads
+
+        :param command: Minecraft command string
+        :param is_after: Whether to add it to after_loads instead
+        """
+        if is_after:
+            self.after_loads.append(command)
+        else:
+            self.loads.append(command)
 
     def add_int(self, integer: int) -> None:
+        """
+        Add command to self.ints
+
+        :param command: Minecraft command string
+        """
         self.ints.add(integer)
 
     def build(self) -> None:
@@ -339,6 +374,8 @@ class DataPack:
         Finializing DataPack for building (NO file writing)
         """
         logger.debug("Finializing DataPack")
+        if self.ints:
+            self.add_objective(self.int_name)
         self.loads[0:0] = [
             *[f"scoreboard objectives add {objective} {criteria}" for objective,
                 criteria in self.__scoreboards.items()],
@@ -346,11 +383,19 @@ class DataPack:
         ]
         if self.loads:
             self.functions[self.load_name].insert_extend(self.loads, 0)
+        if self.after_loads:
+            self.functions[self.load_name].extend(self.after_loads)
         if self.ticks:
             if self.tick_name in self.functions:
                 self.functions[self.tick_name].insert_extend(self.ticks, 0)
             else:
                 self.functions[self.tick_name] = Function(self.ticks)
+        if self.after_ticks:
+            if self.tick_name in self.functions:
+                self.functions[self.tick_name].extend(
+                    self.after_ticks)
+            else:
+                self.functions[self.tick_name] = Function(self.after_ticks)
         for name, functions in self.private_functions.items():
             for path, func in functions.items():
                 self.functions[f"{self.private_name}/{name}/{path}"] = func
@@ -373,14 +418,14 @@ class DataPack:
         for key, value in tokenizer.parse_js_obj(token).items():
             try:
                 num = int(key)
-            except ValueError:
+            except ValueError as error:
                 raise JMCValueError(
-                    f"Expected number as key (got {key})", token, tokenizer)
+                    f"Expected number as key (got {key})", token, tokenizer) from error
 
             if value.token_type == TokenType.KEYWORD:
                 func_map[num] = value.string, False
             elif value.token_type == TokenType.FUNC:
-                func_map[num] = '\n'.join(
+                func_map[num] = "\n".join(
                     self.parse_function_token(value, tokenizer)), True
             else:
                 raise JMCValueError(
@@ -401,8 +446,11 @@ class DataPack:
         token_list = tokenizer.parse_list(token)
         for token_ in token_list:
             if token_.token_type != list_of:
+                token_type = token_.token_type.value
+                if token_.token_type == TokenType.PAREN_SQUARE:
+                    token_type = "list"
                 raise JMCValueError(
-                    f"Expected list/array of {list_of.value}, got {token_.token_type.value}", token, tokenizer)
+                    f"Expected list/array of {list_of.value}, got list/array of {token_type}", token, tokenizer)
         return [token_.string for token_ in token_list]
 
     def token_dict_to_raw_js_object(
@@ -422,7 +470,7 @@ class DataPack:
                     f"{key}:{self.lexer.clean_up_paren_token(token, tokenizer, is_nbt=False)}")
             else:
                 pairs.append(f"{key}:{token.string}")
-        return '{' + ",".join(pairs) + '}'
+        return "{" + ",".join(pairs) + "}"
 
     def __repr__(self) -> str:
         return f"""DataPack(
@@ -439,7 +487,4 @@ class DataPack:
     private_functions = {dumps(self.private_functions, indent=2, cls=FunctionEncoder)}
     loads = {dumps(self.loads, indent=2)}
     tick = {dumps(self.ticks, indent=2)}
-
 )"""
-
-# {dumps({key:list(value) for key, value in self.functions.items()}, indent=2)}
