@@ -1,3 +1,4 @@
+from copy import deepcopy
 from pathlib import Path
 from json import loads, JSONDecodeError, dumps
 from typing import TYPE_CHECKING
@@ -30,11 +31,17 @@ JSON_FILE_TYPES = [
     "worldgen/biome",
     "worldgen/configured_carver",
     "worldgen/configured_feature",
-    "worldgen/configured_structure_feature",
     "worldgen/configured_surface_builder",
+    "worldgen/density_function",
+    "worldgen/flat_level_generator_preset",
+    "worldgen/noise",
     "worldgen/noise_settings",
+    "worldgen/placed_feature",
     "worldgen/processor_list",
+    "worldgen/structure",
+    "worldgen/structure_set",
     "worldgen/template_pool",
+    "worldgen/world_preset"
 ]
 """List of all possible vanilla json file types"""
 
@@ -45,25 +52,46 @@ class Lexer:
 
     :param config: JMC configuration
     """
+    __slots__ = (
+        "if_else_box",
+        "load_tokenizer",
+        "do_while_box",
+        "imports",
+        "config",
+        "datapack")
+
+    if_else_box: list[tuple[Token | None, Token]]
+    """List of tuple of condition(Token) and code block(paren_curly Token) in if-else chain"""
     load_tokenizer: Tokenizer
     """Tokenizer for load function"""
-    do_while_box: Token | None = None
+    do_while_box: Token | None
     """paren_curly token for code block of `do` in `do while`"""
+    imports: set[Path]
+    """Set of path that's already imported"""
+    config: "Configuration"
+    """JMC configuration"""
+    datapack: DataPack
+    """Datapack object"""
 
     def __init__(self, config: "Configuration",
                  _test_file: str | None = None) -> None:
         logger.debug("Initializing Lexer")
-        self.if_else_box: list[tuple[Token | None, Token]] = []
-        """List of tuple of condition(Token) and code block(paren_curly Token) in if-else chain"""
+        self.do_while_box = None
+        self.imports = set()
+        self.if_else_box = []
         self.config = config
-        """JMC configuration"""
         self.datapack = DataPack(config.namespace, self)
-        """Datapack object"""
         self.datapack.functions[self.datapack.load_name] = Function()
         self.parse_file(Path(self.config.target), _test_file, is_load=True)
 
         logger.debug("Load Function")
-        self.parse_current_load()
+
+    def __update_load(self, file_path_str: str, raw_str: str):
+        if file_path_str == self.load_tokenizer.file_path:
+            return
+        self.load_tokenizer.file_path = file_path_str
+        self.load_tokenizer.raw_string = raw_str
+        self.load_tokenizer.file_string = raw_str
 
     def parse_current_load(self):
         """Parse current load function that's in self.datapack.load_function and clear it"""
@@ -90,7 +118,7 @@ class Lexer:
         file_path_str = file_path.resolve().as_posix()
         if _test_file is None:
             try:
-                with file_path.open('r') as file:
+                with file_path.open("r") as file:
                     raw_string = file.read()
             except FileNotFoundError as error:
                 raise JMCFileNotFoundError(
@@ -99,22 +127,43 @@ class Lexer:
             raw_string = _test_file
         tokenizer = Tokenizer(raw_string, file_path_str)
         if is_load:
-            self.load_tokenizer = tokenizer
+            self.load_tokenizer = deepcopy(tokenizer)
+
+        self.__update_load(file_path_str, raw_string)
 
         for command in tokenizer.programs:
-            if command[0].string == 'function' and len(command) == 4:
-                self.parse_func(tokenizer, command, file_path_str)
+            if (
+                command[0].string == "function"
+                and
+                len(command) == 4
+            ):
                 self.parse_current_load()
-            elif command[0].string == 'function' and len(command) != 2:
+                self.parse_func(tokenizer, command, file_path_str)
+            elif (
+                command[0].string == "function"
+                and
+                (
+                    len(command) < 2
+                    or
+                    any(token_.token_type !=
+                        TokenType.KEYWORD for token_ in command[1::2])
+                    or
+                    any(token_.token_type !=
+                        TokenType.OPERATOR for token_ in command[2::2])
+                )
+            ):
                 raise JMCSyntaxException(
-                    f"'function' expect 1(Vanilla syntax) or 3(JMC syntax) arguments (got {len(command)-1})",
+                    "'function' expect Minecraft syntax('namespace:folder/function') or JMC syntax('function name() {}')",
                     command[0],
                     tokenizer)
-            elif command[0].string == 'new':
+            elif command[0].string == "new":
+                self.parse_current_load()
                 self.parse_new(tokenizer, command)
-            elif command[0].string == 'class':
+            elif command[0].string == "class":
+                self.parse_current_load()
                 self.parse_class(tokenizer, command, file_path_str)
-            elif command[0].string == '@import':
+            elif command[0].string == "@import":
+                self.parse_current_load()
                 if len(command) < 2:
                     raise JMCSyntaxException(
                         "Expected string after '@import'", command[0], tokenizer, col_length=True)
@@ -135,23 +184,25 @@ class Lexer:
                         raise JMCFileNotFoundError(
                             f"Directory(folder) not found: {folder.resolve().as_posix()}")
 
-                    new_paths = folder.glob("*.jmc")
+                    new_paths = folder.glob("**/*.jmc")
                     for new_path in new_paths:
                         self.parse_file(file_path=new_path)
-                    return
+                        self.__update_load(file_path_str, raw_string)
+                    continue
                 try:
                     new_path = Path(
                         (file_path.parent / command[1].string).resolve()
                     )
-                    if new_path.suffix != '.jmc':
+                    if new_path.suffix != ".jmc":
                         new_path = Path(
                             (file_path.parent /
-                             (command[1].string + '.jmc')).resolve()
+                             (command[1].string + ".jmc")).resolve()
                         )
                 except Exception as error:
                     raise JMCSyntaxException(
                         f"Unexpected invalid path ({command[1].string})", command[1], tokenizer) from error
                 self.parse_file(file_path=new_path)
+                self.__update_load(file_path_str, raw_string)
             else:
                 # if not is_load:
                 #     raise JMCSyntaxException(
@@ -161,6 +212,8 @@ class Lexer:
 
                 self.datapack.load_function.append(command)
 
+        self.parse_current_load()
+
     def parse_func(self, tokenizer: Tokenizer,
                    command: list[Token], file_path_str: str, prefix: str = '') -> None:
         """
@@ -182,7 +235,7 @@ class Lexer:
         if command[1].token_type != TokenType.KEYWORD:
             raise JMCSyntaxException(
                 "Expected keyword(function's name)", command[1], tokenizer)
-        if command[2].string != '()':
+        if command[2].string != "()":
             raise JMCSyntaxException(
                 "Expected empty round parenthesis", command[2], tokenizer)
         if command[3].token_type != TokenType.PAREN_CURLY:
@@ -190,7 +243,7 @@ class Lexer:
                 "Expected {", command[3], tokenizer, display_col_length=False)
 
         func_path = prefix + convention_jmc_to_mc(command[1], tokenizer)
-        if func_path.startswith(DataPack.private_name + '/'):
+        if func_path.startswith(DataPack.private_name + "/"):
             raise JMCSyntaxException(
                 f"Function({func_path}) may override private function of JMC", command[1], tokenizer, suggestion=f"Please avoid starting function's path with {DataPack.private_name}")
         logger.debug(f"Function: {func_path}")
@@ -243,7 +296,7 @@ class Lexer:
         if len(command) < 3:
             raise JMCSyntaxException(
                 "Expected round parenthesis(JSON file's path)", command[1], tokenizer, col_length=True)
-        if command[2].string == '()':
+        if command[2].string == "()":
             raise JMCSyntaxException(
                 "Expected JSON file's path in the bracket", command[1], tokenizer)
         if command[2].token_type != TokenType.PAREN_ROUND:
@@ -251,10 +304,11 @@ class Lexer:
                 "Expected (", command[1], tokenizer)
         if len(command) < 4:
             raise JMCSyntaxException(
-                "Expected {", command[2], tokenizer, col_length=True)
-        if command[3].token_type != TokenType.PAREN_CURLY:
+                "Expected { or [", command[2], tokenizer, col_length=True)
+        if command[3].token_type not in {
+                TokenType.PAREN_CURLY, TokenType.PAREN_SQUARE}:
             raise JMCSyntaxException(
-                "Expected {", command[3], tokenizer)
+                "Expected { or [", command[3], tokenizer)
 
         json_type = convention_jmc_to_mc(
             command[1], tokenizer, is_make_lower=False)
@@ -268,18 +322,18 @@ class Lexer:
         json_name = prefix + convention_jmc_to_mc(
             command[2], tokenizer, is_make_lower=False, substr=(1, -1))
 
-        if Header().is_override_minecraft and json_name.startswith('minecraft/'):
+        if Header().is_override_minecraft and json_name.startswith("minecraft/"):
             # len('minecraft/') = 10
-            json_path = 'minecraft/' + json_type + '/' + json_name[10:]
+            json_path = "minecraft/" + json_type + "/" + json_name[10:]
         else:
-            json_path = json_type + '/' + json_name
+            json_path = json_type + "/" + json_name
 
         if not json_path.islower():
             raise MinecraftSyntaxWarning(
                 f"Uppercase letter found in JSON file's path({json_path})", command[
                     2], tokenizer
             )
-        if json_path.startswith(DataPack.private_name + '/'):
+        if json_path.startswith(DataPack.private_name + "/"):
             raise JMCSyntaxException(
                 f"JSON({json_path}) may override private function of JMC", command[2], tokenizer, suggestion=f"Please avoid starting JSON's path with {DataPack.private_name}")
 
@@ -332,7 +386,7 @@ class Lexer:
 
         class_path = prefix + convention_jmc_to_mc(command[1], tokenizer)
         class_content = command[2].string[1:-1]
-        self.parse_class_content(class_path + '/',
+        self.parse_class_content(class_path + "/",
                                  class_content, file_path_str, line=command[2].line, col=command[2].col, file_string=tokenizer.file_string)
 
     def parse_load_func_content(
@@ -405,13 +459,13 @@ class Lexer:
         tokenizer = Tokenizer(class_content, file_path_str,
                               line=line, col=col, file_string=file_string)
         for command in tokenizer.programs:
-            if command[0].string == 'function' and len(command) == 4:
+            if command[0].string == "function" and len(command) == 4:
                 self.parse_func(tokenizer, command, file_path_str, prefix)
-            elif command[0].string == 'new':
+            elif command[0].string == "new":
                 self.parse_new(tokenizer, command, prefix)
-            elif command[0].string == 'class':
+            elif command[0].string == "class":
                 self.parse_class(tokenizer, command, file_path_str, prefix)
-            elif command[0].string == '@import':
+            elif command[0].string == "@import":
                 raise JMCSyntaxException(
                     "Importing is not supported in class", command[0], tokenizer)
             else:
@@ -419,7 +473,7 @@ class Lexer:
                     f"Expected 'function' or 'new' or 'class' (got {command[0].string})", command[0], tokenizer)
 
     def parse_if_else(self, tokenizer: Tokenizer,
-                      name: str = 'if_else') -> str:
+                      name: str = "if_else") -> str:
         """
         Parse if-else chain using if_else_box attribute
 
@@ -438,8 +492,12 @@ class Lexer:
             if_else_box[0][0], tokenizer, self.datapack)
         # Case 1: `if` only
         if len(if_else_box) == 1:
-            return_value = f"{precommand}execute {condition} run {self.datapack.add_arrow_function(name, if_else_box[0][1], tokenizer)}"
-            return return_value
+            arrow_func = self.datapack.add_arrow_function(
+                name, if_else_box[0][1], tokenizer)
+            if arrow_func.startswith("execute "):
+                # len('execute ') = 8
+                return f"{precommand}execute {condition} {arrow_func[8:]}"
+            return f"{precommand}execute {condition} run {arrow_func}"
 
         # Case 2: Has `else` or `else if`
         count = self.datapack.get_count(name)
@@ -501,7 +559,7 @@ class Lexer:
         tokenizer = Tokenizer(token.string[1:-1], tokenizer.file_path, token.line,
                               token.col + 1, tokenizer.file_string, expect_semicolon=False, allow_semicolon=token.token_type == TokenType.PAREN_SQUARE)
         string = ""
-        if open_ == '{' and tokenizer.programs[0][0].token_type == TokenType.STRING:
+        if open_ == "{" and tokenizer.programs[0][0].token_type == TokenType.STRING:
             is_nbt = False
         for token_ in tokenizer.programs[0]:
             if token_.token_type == TokenType.PAREN_ROUND:
@@ -517,6 +575,8 @@ class Lexer:
             elif token_.token_type == TokenType.STRING:
                 if is_nbt:
                     _string = repr(token_.string)
+                    if '"' not in _string:
+                        _string = f'"{_string[1:-1]}"'
                 else:
                     _string = dumps(token_.string)
             else:
