@@ -2,6 +2,7 @@
 from typing import TYPE_CHECKING
 from json import dumps
 
+
 from .vanilla_command import COMMANDS as VANILLA_COMMANDS
 from .tokenizer import Tokenizer, Token, TokenType
 from .exception import JMCSyntaxException, MinecraftSyntaxWarning
@@ -9,11 +10,11 @@ from .log import Logger
 from .utils import convention_jmc_to_mc, is_number, is_connected, search_to_string
 from .datapack import DataPack
 from .command.condition import BOOL_FUNCTIONS
+from .header import Header
 from .command import (FLOW_CONTROL_COMMANDS,
                       variable_operation,
                       JMCFunction,
                       FuncType,
-                      JMCFunction,
                       )
 
 if TYPE_CHECKING:
@@ -37,47 +38,60 @@ FIRST_ARGUMENTS = {
     *LOAD_ONLY_COMMANDS,
     *JMC_COMMANDS,
     *FLOW_CONTROL_COMMANDS,
-    *EXECUTE_EXCLUDED_COMMANDS
-} - {'give', 'if'}
+    *EXECUTE_EXCLUDED_COMMANDS,
+    *VANILLA_COMMANDS,
+    "class",
+    "@import",
+    "new"
+}
 """Set of all vanilla commands and JMC custom syntax
 - `if` is excluded since it can be used in execute
 - `give` is exluced since it can also be arguments (`/effect give`)"""
 
-ALLOW_NUMBER_AFTER = [
-    'give',
-    'clear'
-]
-"""List of vanilla command to stop JMC from terminating line from curly parenthesis (Allow number after curly parenthesis)"""
+FIRST_ARGUMENTS_EXCEPTION = {
+    "give": {"effect", "recipe", "give"},
+    "clear": {"effect", "schedule"},
+    "if": {"execute"},
+    "function": {"schedule"},
+    "trigger": {"scoreboard"},
+    "bossbar": {"execute"},
+    "title": {"title"},
+    "item": {"summon"},
+    "loot": {"loot"}
+}
+"""Dictionary of (FIRST_ARGUMENTS that can also be used as normal argument in a command) and (those commands)"""
+
+
+# ALLOW_KEYWORD_AFTER_CURLY_PAREN = {
+#     "give",
+#     "clear",
+#     "setblock"
+# }
+# """Set of vanilla command to stop JMC from terminating line from curly parenthesis (Allow number after curly parenthesis)"""
 
 
 def append_commands(commands: list[str], string: str) -> None:
     """
-    Append a new argument to a comand(list of minecraft arguments) while optimizing it
+    Append a new argument to a comand(list of minecraft arguments)
 
     :param commands: Entire command(list of minecraft arguments) to add to
     :param string: A new argument to add
     """
-    if string.startswith('execute') and commands and commands[-1] == 'run':
-        if string == 'execute':
-            del commands[-1]
-            return
-        else:
-            del commands[-1]
-            commands.append(string[8:])  # len("execute ") = 8
-            return
-    commands.append(string)
-    return
+    if commands and commands[-1] == "run" and string.startswith("execute "):
+        commands[-1] = string[8:]  # len('execute ') = 8
+    else:
+        commands.append(string)
 
 
 class FuncContent:
     """
-    A class representation of a row function for parsing content inside the function
+    A class representation of a raw function for parsing content inside the function
 
     :param tokenizer: Tokenizer
     :param programs: List of commands(List of arguments(Token))
     :param is_load: Whether the function is a load function
     """
-    __slots__ = 'tokenizer', 'programs', 'is_load', 'command_strings', 'commands', 'command', 'is_expect_command', 'is_execute', 'lexer'
+    __slots__ = "tokenizer", "programs", "is_load", "command_strings", "__commands", "command", "is_expect_command", "is_execute", "lexer"
 
     command: list[Token]
     is_expect_command: bool
@@ -90,7 +104,7 @@ class FuncContent:
         self.programs = programs
         self.is_load = is_load
         self.command_strings: list[str] = []
-        self.commands: list[str] = []
+        self.__commands: list[str] = []
         self.lexer = lexer
 
     def parse(self) -> list[str]:
@@ -102,36 +116,36 @@ class FuncContent:
             self.command = command
             if self.command[0].token_type != TokenType.KEYWORD:
                 raise JMCSyntaxException(
-                    "Expected keyword", self.command[0], self.tokenizer)
-            elif self.command[0].string == 'new':
+                    f"Expected keyword (got {self.command[0].token_type.value})", self.command[0], self.tokenizer)
+            if self.command[0].string == "new":
                 raise JMCSyntaxException(
                     "'new' keyword found in function", self.command[0], self.tokenizer)
-            elif self.command[0].string == 'class':
+            if self.command[0].string == "class":
                 raise JMCSyntaxException(
                     "'class' keyword found in function", self.command[0], self.tokenizer)
-            elif self.command[0].string == 'function' and len(self.command) == 4:
+            if self.command[0].string == "function" and len(self.command) == 4:
                 raise JMCSyntaxException(
                     "Function declaration found in function", self.command[0], self.tokenizer)
-            elif self.command[0].string == '@import':
+            if self.command[0].string == "@import":
                 raise JMCSyntaxException(
                     "Importing found in function", self.command[0], self.tokenizer)
 
             # Boxes check
             if self.lexer.do_while_box is not None:
-                if self.command[0].string != 'while':
+                if self.command[0].string != "while":
                     raise JMCSyntaxException(
                         "Expected 'while'", self.command[0], self.tokenizer)
             if self.lexer.if_else_box:
-                if self.command[0].string != 'else':
+                if self.command[0].string != "else":
                     append_commands(
-                        self.commands, self.lexer.parse_if_else(
+                        self.__commands, self.lexer.parse_if_else(
                             self.tokenizer))
 
-            if self.commands:
-                self.command_strings.append(' '.join(self.commands))
-                self.commands = []
+            if self.__commands:
+                self.command_strings.append(" ".join(self.__commands))
+                self.__commands = []
 
-            self.parse_commands()
+            self.__parse_commands()
         # End of Program
 
         # Boxes check
@@ -140,28 +154,82 @@ class FuncContent:
                 "Expected 'while'", self.programs[-1][-1], self.tokenizer)
         if self.lexer.if_else_box:
             append_commands(
-                self.commands,
+                self.__commands,
                 self.lexer.parse_if_else(
                     self.tokenizer))
 
-        if self.commands:
-            self.command_strings.append(' '.join(self.commands))
-            self.commands = []
+        if self.__commands:
+            self.command_strings.append(" ".join(self.__commands))
+            self.__commands = []
         return self.command_strings
 
-    def parse_commands(self) -> None:
+    def __parse_commands(self) -> None:
+        """Parse command in self.commands"""
         self.is_expect_command = True
-        self.is_execute = (self.command[0].string == 'execute')
+        self.is_execute = (self.command[0].string == "execute")
+        command_pos = None
 
         for key_pos, token in enumerate(self.command):
             if not self.is_expect_command:
-                self._not_expect_command(key_pos, token)
+                if command_pos is None:
+                    raise ValueError("Unknown command_pos")
+                self.__not_expect_command(key_pos, token, command_pos)
                 continue
-            if self._expect_command(key_pos, token):
+            command_pos = key_pos
+            if self.__expect_command(key_pos, token):
                 break
 
-    def _not_expect_command(self, key_pos: int, token: Token) -> None:
-        if token.string == 'run' and token.token_type == TokenType.KEYWORD:
+    def __optimize(self, token: Token) -> bool:
+        """
+        Optimize minecraft command by removing redundancy
+        :param token: Current token
+        :return: Whether to cancel appending this token
+        """
+        if token.token_type != TokenType.KEYWORD:
+            return False
+        if not self.__commands:
+            return False
+
+        # `execute as @s`
+        if (
+            token.string == "@s" and
+            self.__commands[-1] == "as" and
+            self.__commands[-2] not in {"rotated", "positioned"}
+        ):
+
+            self.__commands[-1] = "if entity"
+            return False
+
+        # `execute run`
+        if (
+            token.string == "run"
+            and self.__commands[-1] == "execute"
+        ):
+            del self.__commands[-1]
+            return True
+
+        # `run execute`
+        if (
+            token.string == "execute"
+            and self.__commands[-1] == "run"
+            and self.__commands[0] == "execute"
+        ):
+            del self.__commands[-1]
+            return True
+
+        return False
+
+    def __not_expect_command(
+            self, key_pos: int, token: Token, command_pos: int) -> None:
+        """
+        Called when expecting an argument of command
+
+        :param key_pos: Current index in self.command
+        :param token: token
+        :param key_pos: Index of latest command in self.command
+        :return: Whether to break out of the loop
+        """
+        if token.string == "run" and token.token_type == TokenType.KEYWORD:
             if not self.is_execute:
                 raise MinecraftSyntaxWarning(
                     "'run' keyword found outside 'execute' command", token, self.tokenizer)
@@ -169,56 +237,63 @@ class FuncContent:
 
         if (
             token.token_type == TokenType.KEYWORD and
-            token.string in FIRST_ARGUMENTS and
+            (token.string in FIRST_ARGUMENTS or token.string in Header().commands) and
+            len(self.__commands) > command_pos and
             not (
-                token.string == 'function' and
-                self.commands[-1] == 'schedule'
-            )
+                token.string in FIRST_ARGUMENTS_EXCEPTION
+                and
+                self.__commands[command_pos] in FIRST_ARGUMENTS_EXCEPTION[token.string]
+            ) and
+            not is_connected(token, self.command[key_pos - 1])
         ):
             raise JMCSyntaxException(
                 f"Keyword({token.string}) at line {token.line} col {token.col} is recognized as a command.\nExpected semicolon(;)", self.command[key_pos - 1], self.tokenizer, col_length=True)
 
-        # optimize `execute as @s`
-        if (token.string == '@s' and
-            token.token_type == TokenType.KEYWORD and
-            self.commands[-1] == 'as' and
-                self.commands[-2] not in {'rotated', 'positioned'}):
-
-            self.commands[-1] = 'if entity'
+        if self.__optimize(token):
+            return
 
         if token.token_type == TokenType.PAREN_ROUND:
-            self.commands[-1], success = search_to_string(
-                self.commands[-1], token, DataPack.var_name, self.tokenizer)
+            self.__commands[-1], success = search_to_string(
+                self.__commands[-1], token, DataPack.var_name, self.tokenizer)
             if not success:
                 if is_connected(token, self.command[key_pos - 1]):
-                    self.commands[-1] += self.lexer.clean_up_paren_token(
+                    self.__commands[-1] += self.lexer.clean_up_paren_token(
                         token, self.tokenizer)
                 else:
                     append_commands(
-                        self.commands, self.lexer.clean_up_paren_token(token, self.tokenizer))
+                        self.__commands, self.lexer.clean_up_paren_token(token, self.tokenizer))
         elif token.token_type in {TokenType.PAREN_CURLY, TokenType.PAREN_SQUARE}:
             if is_connected(token, self.command[key_pos - 1]):
-                self.commands[-1] += self.lexer.clean_up_paren_token(
+                self.__commands[-1] += self.lexer.clean_up_paren_token(
                     token, self.tokenizer)
             else:
                 append_commands(
-                    self.commands, self.lexer.clean_up_paren_token(token, self.tokenizer))
+                    self.__commands, self.lexer.clean_up_paren_token(token, self.tokenizer))
         elif token.token_type == TokenType.STRING:
-            append_commands(self.commands, dumps(token.string))
+            append_commands(self.__commands, dumps(token.string))
+        elif token.token_type in {TokenType.OPERATOR, TokenType.KEYWORD} and is_connected(token, self.command[key_pos - 1]):
+            self.__commands[-1] += token.string
         else:
-            append_commands(self.commands, token.string)
+            append_commands(self.__commands, token.string)
 
-    def _expect_command(self, key_pos: int, token: Token) -> bool:
+    def __expect_command(self, key_pos: int, token: Token) -> bool:
+        """
+        Called when expecting a command
+
+        :param key_pos: Current index in self.command
+        :param token: token
+        :return: Whether to break out of the loop
+        """
         self.is_expect_command = False
+
         # Handle Errors
         if token.token_type != TokenType.KEYWORD:
             if token.token_type == TokenType.PAREN_CURLY and self.is_execute:
-                append_commands(self.commands, self.lexer.datapack.add_arrow_function(
-                    'anonymous', token, self.tokenizer))
+                append_commands(self.__commands, self.lexer.datapack.add_arrow_function(
+                    "anonymous", token, self.tokenizer))
                 return True
-            else:
-                raise JMCSyntaxException(
-                    "Expected keyword", token, self.tokenizer)
+            raise JMCSyntaxException(
+                "Expected keyword", token, self.tokenizer)
 
         # End Handle Errors
 
@@ -226,10 +301,14 @@ class FuncContent:
             self.__is_number(key_pos, token)
             return True
 
-        if token.string == 'say':
+        if token.string == "say":
             self.__is_say(key_pos, token)
             return True
 
+        if token.string == "schedule":
+            if self.__is_schedule(key_pos, token):
+                return True
+
         if token.string.startswith(DataPack.VARIABLE_SIGN):
             if self.__is_startswith_varsign(key_pos, token):
                 return True
@@ -243,10 +322,12 @@ class FuncContent:
         if self.__is_flow_control_command(key_pos, token):
             return True
 
-        if token.string in {'new', 'class' '@import'} or (
-            token.string == 'function'
+        if token.string in {"new", "class", "@import"} or (
+            token.string == "function"
             and
-            len(self.command) > key_pos + 2
+            len(self.command) == key_pos + 4
+            and self.command[key_pos + 2].token_type == TokenType.PAREN_ROUND
+            and self.command[key_pos + 3].token_type == TokenType.PAREN_CURLY
         ):
             if self.is_execute:
                 raise JMCSyntaxException(
@@ -257,18 +338,37 @@ class FuncContent:
                 raise JMCSyntaxException(
                     f"Unexpected token({self.command[key_pos+2].string}) after function call. Expected semicolon(;)", self.command[key_pos + 1], self.tokenizer, col_length=True)
 
-            if self.command[key_pos + 1].string != '()':
+            if self.command[key_pos + 1].string != "()":
                 raise JMCSyntaxException(
-                    f"Custom function({token.string})'s parameter is not supported.\nExpected empty bracket", self.command[key_pos + 1], self.tokenizer)
-            append_commands(self.commands,
+                    f"Custom function({token.string})'s argument is not supported.\nExpected empty bracket", self.command[key_pos + 1], self.tokenizer)
+            append_commands(self.__commands,
                             f"function {self.lexer.datapack.namespace}:{convention_jmc_to_mc(token, self.tokenizer)}")
             return True
 
-        if token.string not in VANILLA_COMMANDS:
+        if token.string not in VANILLA_COMMANDS and token.string not in Header().commands:
+            if not self.command_strings:
+                raise JMCSyntaxException(
+                    f"Unrecognized command ({token.string})", token, self.tokenizer)
+
+            # if not self.command_strings[-1].startswith("execute"):
+            #     # for _cmd in ALLOW_KEYWORD_AFTER_CURLY_PAREN:
+            #     #     if self.command_strings[-1].startswith(_cmd):
+            #     #         self.command_strings[-1] += " " + token.string
+            #     #         return True
+            #     raise JMCSyntaxException(
+            # f"Unrecognized command ({token.string})", token, self.tokenizer)
+
+            # for _cmd in ALLOW_KEYWORD_AFTER_CURLY_PAREN:
+            #     if _cmd in self.command_strings[-1]:
+            #         self.command_strings[-1] += " " + token.string
+            #         return True
+
             raise JMCSyntaxException(
                 f"Unrecognized command ({token.string})", token, self.tokenizer)
 
-        append_commands(self.commands, token.string)
+        if self.__optimize(token):
+            return False
+        append_commands(self.__commands, token.string)
         return False
 
     def __is_number(self, key_pos: int, token: Token) -> None:
@@ -280,18 +380,18 @@ class FuncContent:
             raise JMCSyntaxException(
                 "Expected command, got number", token, self.tokenizer, display_col_length=False)
 
-        if not self.command_strings[-1].startswith('execute'):
-            for _cmd in ALLOW_NUMBER_AFTER:
-                if self.command_strings[-1].startswith(_cmd):
-                    self.command_strings[-1] += ' ' + token.string
-                    return
-            raise JMCSyntaxException(
-                "Unexpected number", token, self.tokenizer, display_col_length=False)
+        # if not self.command_strings[-1].startswith("execute"):
+        #     for _cmd in ALLOW_KEYWORD_AFTER_CURLY_PAREN:
+        #         if self.command_strings[-1].startswith(_cmd):
+        #             self.command_strings[-1] += " " + token.string
+        #             return
+        #     raise JMCSyntaxException(
+        #         "Unexpected number", token, self.tokenizer, display_col_length=False)
 
-        for _cmd in ALLOW_NUMBER_AFTER:
-            if _cmd in self.command_strings[-1]:
-                self.command_strings[-1] += ' ' + token.string
-                return
+        # for _cmd in ALLOW_KEYWORD_AFTER_CURLY_PAREN:
+        #     if _cmd in self.command_strings[-1]:
+        #         self.command_strings[-1] += " " + token.string
+        #         return
 
         raise JMCSyntaxException(
             "Unexpected number", token, self.tokenizer, display_col_length=False)
@@ -309,24 +409,89 @@ class FuncContent:
             raise JMCSyntaxException(
                 "Unexpected token", self.command[key_pos + 2], self.tokenizer, display_col_length=False)
 
-        if '\n' in self.command[key_pos + 1].string:
+        if "\n" in self.command[key_pos + 1].string:
             raise JMCSyntaxException(
-                "Newline found in say command", self.command[key_pos + 1], self.tokenizer)
+                "Newline found in say command", self.command[key_pos + 1], self.tokenizer, suggestion=r"Use '\\n' instead of '\n'")
         append_commands(
-            self.commands, f"say {self.command[key_pos+1].string}")
+            self.__commands, f"say {self.command[key_pos+1].string}")
+
+    def __is_schedule(self, key_pos: int, token: Token) -> bool:
+        if len(self.command) < key_pos + 3:
+            return False
+        if (
+            self.command[key_pos + 2].token_type == TokenType.PAREN_CURLY
+        ):
+            # `schedule 1t {say "command";}`
+            append_commands(self.__commands, "schedule")
+            append_commands(self.__commands, self.lexer.datapack.add_arrow_function(
+                "anonymous", self.command[key_pos + 2], self.tokenizer, force_create_func=True))
+            append_commands(self.__commands, self.command[key_pos + 1].string)
+            return True
+        if len(self.command) < key_pos + 4:
+            return False
+        if (
+            self.command[key_pos + 2].token_type == TokenType.KEYWORD
+            and
+            self.command[key_pos + 3].token_type == TokenType.PAREN_ROUND
+        ):
+            # `schedule function myFunc() 1t append;`
+            if self.command[key_pos + 1].string not in {"function", "clear"}:
+                raise JMCSyntaxException(
+                    "Expected 'function' or 'clear' after 'schedule'", self.command[key_pos + 1], self.tokenizer)
+            if self.command[key_pos + 3].string != "()":
+                raise JMCSyntaxException(
+                    f"'schedule' does not support argument is not supported.\nExpected empty bracket", self.command[key_pos + 3], self.tokenizer)
+            append_commands(self.__commands, "schedule")
+            append_commands(self.__commands, self.command[key_pos + 1].string)
+            append_commands(
+                self.__commands,
+                f"{self.lexer.datapack.namespace}:{convention_jmc_to_mc(self.command[key_pos + 2], self.tokenizer)}")
+            if self.command[key_pos + 1].string == "clear":
+                if len(self.command) > key_pos + 4:
+                    raise JMCSyntaxException(
+                        "Unexpected token in schedule clear", self.command[key_pos + 3], self.tokenizer)
+                return True
+            if len(self.command) < key_pos + 5:
+                raise JMCSyntaxException(
+                    f"Expected time in schedule call", self.command[key_pos + 4], self.tokenizer)
+            append_commands(self.__commands, self.command[key_pos + 4].string)
+            if len(self.command) == key_pos + 6:
+                if self.command[key_pos +
+                                5].string not in {"append", "replace"}:
+                    raise JMCSyntaxException(
+                        f"Expected 'append' or 'replace' (got {self.command[key_pos + 5]}", self.command[key_pos + 5], self.tokenizer)
+                append_commands(self.__commands,
+                                self.command[key_pos + 5].string)
+            if len(self.command) > key_pos + 6:
+                raise JMCSyntaxException(
+                    "Unexpected token in schedule", self.command[key_pos + 5], self.tokenizer)
+            return True
+        if (
+            self.command[key_pos + 3].token_type == TokenType.PAREN_CURLY
+        ):
+            # `schedule 1t append {say "command";}`
+            if self.command[key_pos +
+                            2].string not in {"append", "replace"}:
+                raise JMCSyntaxException(
+                    f"Expected 'append' or 'replace' (got {self.command[key_pos + 2]}", self.command[key_pos + 2], self.tokenizer)
+            append_commands(self.__commands, "schedule")
+            append_commands(self.__commands, self.lexer.datapack.add_arrow_function(
+                "anonymous", self.command[key_pos + 3], self.tokenizer, force_create_func=True))
+            append_commands(self.__commands, self.command[key_pos + 1].string)
+            append_commands(self.__commands, self.command[key_pos + 2].string)
+            return True
+        return False
 
     def __is_startswith_varsign(self, key_pos: int, token: Token) -> bool:
         if len(
-                self.command) > key_pos + 1 and self.command[key_pos + 1].string == 'run' and self.command[key_pos + 1].token_type == TokenType.KEYWORD:
+                self.command) > key_pos + 1 and self.command[key_pos + 1].string == "run" and self.command[key_pos + 1].token_type == TokenType.KEYWORD:
             self.is_execute = True
-            append_commands(self.commands,
+            append_commands(self.__commands,
                             f"execute store result score {token.string} {DataPack.var_name}")
             return False
-
-        else:
-            append_commands(self.commands, variable_operation(
-                self.command[key_pos:], self.tokenizer, self.lexer.datapack, self.is_execute))
-            return True
+        append_commands(self.__commands, variable_operation(
+            self.command[key_pos:], self.tokenizer, self.lexer.datapack, self.is_execute, type(self), FIRST_ARGUMENTS))
+        return True
 
     def __is_jmc_function(self, key_pos: int, token: Token) -> bool:
         load_once_command = self.get_function(token, LOAD_ONCE_COMMANDS)
@@ -342,7 +507,7 @@ class FuncContent:
                     f"This feature({token.string}) can only be used in load function", token, self.tokenizer)
 
             self.lexer.datapack.used_command.add(token.string)
-            append_commands(self.commands, load_once_command(
+            append_commands(self.__commands, load_once_command(
                 self.command[key_pos + 1], self.lexer.datapack, self.tokenizer).call())
             return True
 
@@ -353,7 +518,7 @@ class FuncContent:
                 raise JMCSyntaxException(
                     f"This feature({token.string}) cannot be used with 'execute'", token, self.tokenizer)
             self.lexer.datapack.used_command.add(token.string)
-            append_commands(self.commands, execute_excluded_command(
+            append_commands(self.__commands, execute_excluded_command(
                 self.command[key_pos + 1], self.lexer.datapack, self.tokenizer).call())
             return True
 
@@ -365,7 +530,7 @@ class FuncContent:
             if not self.is_load:
                 raise JMCSyntaxException(
                     f"This feature({token.string}) can only be used in load function", token, self.tokenizer)
-            append_commands(self.commands, load_only_command(
+            append_commands(self.__commands, load_only_command(
                 self.command[key_pos + 1], self.lexer.datapack, self.tokenizer).call())
             return True
 
@@ -374,7 +539,7 @@ class FuncContent:
             if len(self.command) > key_pos + 2:
                 raise JMCSyntaxException(
                     "Unexpected token", self.command[key_pos + 2], self.tokenizer, display_col_length=False)
-            append_commands(self.commands, jmc_command(
+            append_commands(self.__commands, jmc_command(
                 self.command[key_pos + 1], self.lexer.datapack, self.tokenizer, is_execute=self.is_execute).call())
             return True
 
@@ -394,10 +559,15 @@ class FuncContent:
             return_value = flow_control_command(
                 self.command[key_pos:], self.lexer.datapack, self.tokenizer)
             if return_value is not None:
-                append_commands(self.commands, return_value)
+                append_commands(self.__commands, return_value)
             return True
         return False
 
     def get_function(self, token: Token,
                      command_functions: dict[str, type["JMCFunction"]]) -> type["JMCFunction"] | None:
+        """
+        Get jmc function (class)
+
+        :return: The JMCFunction's subclass
+        """
         return command_functions.get(token.string, None)
